@@ -39,6 +39,7 @@
 #include "mm-broadband-bearer-mbm.h"
 #include "mm-log.h"
 #include "mm-modem-helpers.h"
+#include "mm-daemon-enums-types.h"
 
 G_DEFINE_TYPE (MMBroadbandBearerMbm, mm_broadband_bearer_mbm, MM_TYPE_BROADBAND_BEARER);
 
@@ -57,46 +58,23 @@ typedef struct {
     MMAtSerialPort *primary;
     guint cid;
     GCancellable *cancellable;
+    MMPort *data;
     GSimpleAsyncResult *result;
     guint poll_count;
 } Dial3gppContext;
 
-static Dial3gppContext *
-dial_3gpp_context_new (MMBroadbandBearerMbm *self,
-                       MMBaseModem *modem,
-                       MMAtSerialPort *primary,
-                       guint cid,
-                       GCancellable *cancellable,
-                       GAsyncReadyCallback callback,
-                       gpointer user_data)
-{
-    Dial3gppContext *ctx;
-
-    ctx = g_new0 (Dial3gppContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->modem = g_object_ref (modem);
-    ctx->primary = g_object_ref (primary);
-    ctx->cid = cid;
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             dial_3gpp_context_new);
-    ctx->cancellable = g_object_ref (cancellable);
-    ctx->poll_count = 0;
-
-    return ctx;
-}
-
 static void
 dial_3gpp_context_complete_and_free (Dial3gppContext *ctx)
 {
-    g_simple_async_result_complete (ctx->result);
+    g_simple_async_result_complete_in_idle (ctx->result);
+    if (ctx->data)
+        g_object_unref (ctx->data);
     g_object_unref (ctx->cancellable);
     g_object_unref (ctx->result);
     g_object_unref (ctx->primary);
     g_object_unref (ctx->modem);
     g_object_unref (ctx->self);
-    g_free (ctx);
+    g_slice_free (Dial3gppContext, ctx);
 }
 
 static gboolean
@@ -126,61 +104,74 @@ dial_3gpp_context_complete_and_free_if_cancelled (Dial3gppContext *ctx)
     return TRUE;
 }
 
-static gboolean
+static MMPort *
 dial_3gpp_finish (MMBroadbandBearer *self,
                   GAsyncResult *res,
                   GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+        return NULL;
+
+    return MM_PORT (g_object_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res))));
 }
 
-void
-mm_broadband_bearer_mbm_report_connection_status (MMBroadbandBearerMbm *self,
-                                                  MMBroadbandBearerMbmConnectionStatus status)
+static void
+report_connection_status (MMBearer *bearer,
+                          MMBearerConnectionStatus status)
 {
+    MMBroadbandBearerMbm *self = MM_BROADBAND_BEARER_MBM (bearer);
     Dial3gppContext *ctx;
 
+    g_assert (status == MM_BEARER_CONNECTION_STATUS_CONNECTED ||
+              status == MM_BEARER_CONNECTION_STATUS_DISCONNECTED);
+
     /* Recover context (if any) and remove both cancellation and timeout (if any)*/
     ctx = self->priv->connect_pending;
     self->priv->connect_pending = NULL;
 
+    /* Connection status reported but no connection attempt? */
+    if (!ctx) {
+        g_assert (self->priv->connect_pending_id == 0);
+
+        mm_dbg ("Received spontaneous *E2NAP (%s)",
+                mm_bearer_connection_status_get_string (status));
+
+        if (status == MM_BEARER_CONNECTION_STATUS_DISCONNECTED) {
+            /* If no connection attempt on-going, make sure we mark ourselves as
+             * disconnected */
+            MM_BEARER_CLASS (mm_broadband_bearer_mbm_parent_class)->report_connection_status (
+                bearer,
+                status);
+        }
+        return;
+    }
+
     if (self->priv->connect_pending_id) {
         g_source_remove (self->priv->connect_pending_id);
         self->priv->connect_pending_id = 0;
     }
 
-    if (ctx && self->priv->connect_cancellable_id) {
+    if (self->priv->connect_cancellable_id) {
         g_cancellable_disconnect (ctx->cancellable,
                                   self->priv->connect_cancellable_id);
         self->priv->connect_cancellable_id = 0;
     }
 
-    switch (status) {
-    case MM_BROADBAND_BEARER_MBM_CONNECTION_STATUS_UNKNOWN:
-        g_warn_if_reached ();
-        break;
-
-    case MM_BROADBAND_BEARER_MBM_CONNECTION_STATUS_CONNECTED:
-        if (!ctx)
-            break;
-
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+    /* Reporting connected */
+    if (status == MM_BEARER_CONNECTION_STATUS_CONNECTED) {
+        g_simple_async_result_set_op_res_gpointer (ctx->result,
+                                                   g_object_ref (ctx->data),
+                                                   (GDestroyNotify)g_object_unref);
         dial_3gpp_context_complete_and_free (ctx);
         return;
-
-    case MM_BROADBAND_BEARER_MBM_CONNECTION_STATUS_DISCONNECTED:
-        if (ctx) {
-            g_simple_async_result_set_error (ctx->result,
-                                             MM_CORE_ERROR,
-                                             MM_CORE_ERROR_FAILED,
-                                             "Call setup failed");
-            dial_3gpp_context_complete_and_free (ctx);
-        } else {
-            /* Just ensure we mark ourselves as being disconnected... */
-            mm_bearer_report_disconnection (MM_BEARER (self));
-        }
-        break;
     }
+
+    /* Reporting disconnected */
+    g_simple_async_result_set_error (ctx->result,
+                                     MM_CORE_ERROR,
+                                     MM_CORE_ERROR_FAILED,
+                                     "Call setup failed");
+    dial_3gpp_context_complete_and_free (ctx);
 }
 
 static void
@@ -246,7 +237,9 @@ poll_ready (MMBaseModem *modem,
             self->priv->connect_cancellable_id = 0;
         }
 
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+        g_simple_async_result_set_op_res_gpointer (ctx->result,
+                                                   g_object_ref (ctx->data),
+                                                   (GDestroyNotify)g_object_unref);
         dial_3gpp_context_complete_and_free (ctx);
         return;
     }
@@ -410,8 +403,10 @@ authenticate (Dial3gppContext *ctx)
 
 		command = g_strdup_printf ("AT*EIAAUW=%d,1,\"%s\",\"%s\"",
                                    ctx->cid,
-                                   encoded_user,
-                                   encoded_password);
+                                   encoded_user ? encoded_user : "",
+                                   encoded_password ? encoded_password : "");
+        g_free (encoded_user);
+        g_free (encoded_password);
 
         mm_base_modem_at_command_full (ctx->modem,
                                        ctx->primary,
@@ -426,6 +421,7 @@ authenticate (Dial3gppContext *ctx)
         return;
     }
 
+    mm_dbg ("Authentication not needed");
     activate (ctx);
 }
 
@@ -433,21 +429,40 @@ static void
 dial_3gpp (MMBroadbandBearer *self,
            MMBaseModem *modem,
            MMAtSerialPort *primary,
-           MMPort *data, /* unused by us */
            guint cid,
            GCancellable *cancellable,
            GAsyncReadyCallback callback,
            gpointer user_data)
 {
+    Dial3gppContext *ctx;
+
     g_assert (primary != NULL);
 
-    authenticate (dial_3gpp_context_new (MM_BROADBAND_BEARER_MBM (self),
-                                         modem,
-                                         primary,
-                                         cid,
-                                         cancellable,
-                                         callback,
-                                         user_data));
+    ctx = g_slice_new0 (Dial3gppContext);
+    ctx->self = g_object_ref (self);
+    ctx->modem = g_object_ref (modem);
+    ctx->primary = g_object_ref (primary);
+    ctx->cid = cid;
+    ctx->result = g_simple_async_result_new (G_OBJECT (self),
+                                             callback,
+                                             user_data,
+                                             dial_3gpp);
+    ctx->cancellable = g_object_ref (cancellable);
+    ctx->poll_count = 0;
+
+    /* We need a net data port */
+    ctx->data = mm_base_modem_get_best_data_port (modem, MM_PORT_TYPE_NET);
+    if (!ctx->data) {
+        g_simple_async_result_set_error (
+            ctx->result,
+            MM_CORE_ERROR,
+            MM_CORE_ERROR_NOT_FOUND,
+            "No valid data port found to launch connection");
+        dial_3gpp_context_complete_and_free (ctx);
+        return;
+    }
+
+    authenticate (ctx);
 }
 
 /*****************************************************************************/
@@ -585,10 +600,12 @@ mm_broadband_bearer_mbm_class_init (MMBroadbandBearerMbmClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+    MMBearerClass *bearer_class = MM_BEARER_CLASS (klass);
     MMBroadbandBearerClass *broadband_bearer_class = MM_BROADBAND_BEARER_CLASS (klass);
 
     g_type_class_add_private (object_class, sizeof (MMBroadbandBearerMbmPrivate));
 
+    bearer_class->report_connection_status = report_connection_status;
     broadband_bearer_class->dial_3gpp = dial_3gpp;
     broadband_bearer_class->dial_3gpp_finish = dial_3gpp_finish;
     broadband_bearer_class->disconnect_3gpp = disconnect_3gpp;
