@@ -37,6 +37,7 @@
 #include "mm-modem-helpers.h"
 #include "mm-broadband-modem-mbm.h"
 #include "mm-broadband-bearer-mbm.h"
+#include "mm-sim-mbm.h"
 #include "mm-base-modem-at.h"
 #include "mm-iface-modem.h"
 #include "mm-iface-modem-3gpp.h"
@@ -44,23 +45,24 @@
 static void iface_modem_init (MMIfaceModem *iface);
 static void iface_modem_3gpp_init (MMIfaceModem3gpp *iface);
 
+static MMIfaceModem *iface_modem_parent;
 static MMIfaceModem3gpp *iface_modem_3gpp_parent;
 
 G_DEFINE_TYPE_EXTENDED (MMBroadbandModemMbm, mm_broadband_modem_mbm, MM_TYPE_BROADBAND_MODEM, 0,
                         G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM, iface_modem_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_IFACE_MODEM_3GPP, iface_modem_3gpp_init))
 
-#define MBM_NETWORK_MODE_ANY  1
-#define MBM_NETWORK_MODE_2G   5
-#define MBM_NETWORK_MODE_3G   6
+#define MBM_NETWORK_MODE_OFFLINE   0
+#define MBM_NETWORK_MODE_ANY       1
+#define MBM_NETWORK_MODE_LOW_POWER 4
+#define MBM_NETWORK_MODE_2G        5
+#define MBM_NETWORK_MODE_3G        6
 
 #define MBM_E2NAP_DISCONNECTED 0
 #define MBM_E2NAP_CONNECTED    1
 #define MBM_E2NAP_CONNECTING   2
 
 struct _MMBroadbandModemMbmPrivate {
-    guint network_mode;
-
     gboolean have_emrdy;
 
     GRegex *e2nap_regex;
@@ -68,8 +70,11 @@ struct _MMBroadbandModemMbmPrivate {
     GRegex *emrdy_regex;
     GRegex *pacsp_regex;
     GRegex *estksmenu_regex;
+    GRegex *estksms_regex;
     GRegex *emwi_regex;
     GRegex *erinfo_regex;
+
+    guint mbm_mode;
 };
 
 /*****************************************************************************/
@@ -128,6 +133,29 @@ modem_create_bearer (MMIfaceModem *self,
                                  result);
 }
 
+/*****************************************************************************/
+/* Create SIM (Modem interface) */
+
+static MMSim *
+create_sim_finish (MMIfaceModem *self,
+                   GAsyncResult *res,
+                   GError **error)
+{
+    return mm_sim_mbm_new_finish (res, error);
+}
+
+static void
+create_sim (MMIfaceModem *self,
+            GAsyncReadyCallback callback,
+            gpointer user_data)
+{
+    /* New MBM SIM */
+    mm_sim_mbm_new (MM_BASE_MODEM (self),
+                    NULL, /* cancellable */
+                    callback,
+                    user_data);
+}
+
 /*****************************************************************************/
 /* After SIM unlock (Modem interface) */
 
@@ -163,16 +191,91 @@ modem_after_sim_unlock (MMIfaceModem *self,
     g_timeout_add (500, (GSourceFunc)after_sim_unlock_wait_cb, result);
 }
 
+/*****************************************************************************/
+/* Load supported modes (Modem interface) */
+
+static GArray *
+load_supported_modes_finish (MMIfaceModem *self,
+                             GAsyncResult *res,
+                             GError **error)
+{
+    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
+        return NULL;
+
+    return g_array_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
+}
+
+static void
+parent_load_supported_modes_ready (MMIfaceModem *self,
+                                   GAsyncResult *res,
+                                   GSimpleAsyncResult *simple)
+{
+    GError *error = NULL;
+    GArray *all;
+    GArray *combinations;
+    GArray *filtered;
+    MMModemModeCombination mode;
+
+    all = iface_modem_parent->load_supported_modes_finish (self, res, &error);
+    if (!all) {
+        g_simple_async_result_take_error (simple, error);
+        g_simple_async_result_complete (simple);
+        g_object_unref (simple);
+        return;
+    }
+
+    /* Build list of combinations */
+    combinations = g_array_sized_new (FALSE, FALSE, sizeof (MMModemModeCombination), 3);
+
+    /* 2G only */
+    mode.allowed = MM_MODEM_MODE_2G;
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+    /* 3G only */
+    mode.allowed = MM_MODEM_MODE_3G;
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+    /* 2G and 3G */
+    mode.allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
+    mode.preferred = MM_MODEM_MODE_NONE;
+    g_array_append_val (combinations, mode);
+
+    /* Filter out those unsupported modes */
+    filtered = mm_filter_supported_modes (all, combinations);
+    g_array_unref (all);
+    g_array_unref (combinations);
+
+    g_simple_async_result_set_op_res_gpointer (simple, filtered, (GDestroyNotify) g_array_unref);
+    g_simple_async_result_complete (simple);
+    g_object_unref (simple);
+}
+
+static void
+load_supported_modes (MMIfaceModem *self,
+                      GAsyncReadyCallback callback,
+                      gpointer user_data)
+{
+    /* Run parent's loading */
+    iface_modem_parent->load_supported_modes (
+        MM_IFACE_MODEM (self),
+        (GAsyncReadyCallback)parent_load_supported_modes_ready,
+        g_simple_async_result_new (G_OBJECT (self),
+                                   callback,
+                                   user_data,
+                                   load_supported_modes));
+}
+
 /*****************************************************************************/
 /* Load initial allowed/preferred modes (Modem interface) */
 
 static gboolean
-load_allowed_modes_finish (MMIfaceModem *self,
+load_current_modes_finish (MMIfaceModem *_self,
                            GAsyncResult *res,
                            MMModemMode *allowed,
                            MMModemMode *preferred,
                            GError **error)
 {
+    MMBroadbandModemMbm *self = MM_BROADBAND_MODEM_MBM (_self);
     const gchar *response;
     guint a;
 
@@ -180,18 +283,26 @@ load_allowed_modes_finish (MMIfaceModem *self,
     if (!response)
         return FALSE;
 
-    if (mm_get_uint_from_str (mm_strip_tag (response, "CFUN:"), &a)) {
+    if (mm_get_uint_from_str (mm_strip_tag (response, "+CFUN:"), &a)) {
         /* No settings to set preferred */
         *preferred = MM_MODEM_MODE_NONE;
 
         switch (a) {
+        case MBM_NETWORK_MODE_OFFLINE:
+        case MBM_NETWORK_MODE_LOW_POWER:
+            /* Do not update internal mbm_mode */
+            *allowed = MM_MODEM_MODE_NONE;
+            break;
         case MBM_NETWORK_MODE_2G:
+            self->priv->mbm_mode = MBM_NETWORK_MODE_2G;
             *allowed = MM_MODEM_MODE_2G;
             break;
         case MBM_NETWORK_MODE_3G:
+            self->priv->mbm_mode = MBM_NETWORK_MODE_3G;
             *allowed = MM_MODEM_MODE_3G;
             break;
         default:
+            /* Do not update internal mbm_mode */
             *allowed = (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G);
             break;
         }
@@ -208,7 +319,7 @@ load_allowed_modes_finish (MMIfaceModem *self,
 }
 
 static void
-load_allowed_modes (MMIfaceModem *self,
+load_current_modes (MMIfaceModem *self,
                     GAsyncReadyCallback callback,
                     gpointer user_data)
 {
@@ -223,8 +334,23 @@ load_allowed_modes (MMIfaceModem *self,
 /*****************************************************************************/
 /* Set allowed modes (Modem interface) */
 
+typedef struct {
+    MMBroadbandModemMbm *self;
+    GSimpleAsyncResult *result;
+    gint mbm_mode;
+} SetCurrentModesContext;
+
+static void
+set_current_modes_context_complete_and_free (SetCurrentModesContext *ctx)
+{
+    g_simple_async_result_complete_in_idle (ctx->result);
+    g_object_unref (ctx->result);
+    g_object_unref (ctx->self);
+    g_slice_free (SetCurrentModesContext, ctx);
+}
+
 static gboolean
-set_allowed_modes_finish (MMIfaceModem *self,
+set_current_modes_finish (MMIfaceModem *self,
                           GAsyncResult *res,
                           GError **error)
 {
@@ -234,53 +360,56 @@ set_allowed_modes_finish (MMIfaceModem *self,
 static void
 allowed_mode_update_ready (MMBaseModem *self,
                            GAsyncResult *res,
-                           GSimpleAsyncResult *operation_result)
+                           SetCurrentModesContext *ctx)
 {
     GError *error = NULL;
 
     mm_base_modem_at_command_finish (self, res, &error);
     if (error)
         /* Let the error be critical. */
-        g_simple_async_result_take_error (operation_result, error);
-    else
-        g_simple_async_result_set_op_res_gboolean (operation_result, TRUE);
-    g_simple_async_result_complete (operation_result);
-    g_object_unref (operation_result);
+        g_simple_async_result_take_error (ctx->result, error);
+    else {
+        /* Cache current allowed mode */
+        ctx->self->priv->mbm_mode = ctx->mbm_mode;
+        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+    }
+    set_current_modes_context_complete_and_free (ctx);
 }
 
 static void
-set_allowed_modes (MMIfaceModem *_self,
+set_current_modes (MMIfaceModem *self,
                    MMModemMode allowed,
                    MMModemMode preferred,
                    GAsyncReadyCallback callback,
                    gpointer user_data)
 {
-    MMBroadbandModemMbm *self = MM_BROADBAND_MODEM_MBM (_self);
-    GSimpleAsyncResult *result;
+    SetCurrentModesContext *ctx;
     gchar *command;
-    gint mbm_mode = -1;
 
-    result = g_simple_async_result_new (G_OBJECT (self),
-                                        callback,
-                                        user_data,
-                                        set_allowed_modes);
+    ctx = g_slice_new (SetCurrentModesContext);
+    ctx->self = g_object_ref (self);
+    ctx->result = g_simple_async_result_new (G_OBJECT (self),
+                                             callback,
+                                             user_data,
+                                             set_current_modes);
+    ctx->mbm_mode = -1;
 
     if (allowed == MM_MODEM_MODE_2G)
-        mbm_mode = MBM_NETWORK_MODE_2G;
+        ctx->mbm_mode = MBM_NETWORK_MODE_2G;
     else if (allowed == MM_MODEM_MODE_3G)
-        mbm_mode = MBM_NETWORK_MODE_3G;
+        ctx->mbm_mode = MBM_NETWORK_MODE_3G;
     else if ((allowed == (MM_MODEM_MODE_2G | MM_MODEM_MODE_3G) ||
               allowed == MM_MODEM_MODE_ANY) &&
              preferred == MM_MODEM_MODE_NONE)
-        mbm_mode = MBM_NETWORK_MODE_ANY;
+        ctx->mbm_mode = MBM_NETWORK_MODE_ANY;
 
-    if (mbm_mode < 0) {
+    if (ctx->mbm_mode < 0) {
         gchar *allowed_str;
         gchar *preferred_str;
 
         allowed_str = mm_modem_mode_build_string_from_mask (allowed);
         preferred_str = mm_modem_mode_build_string_from_mask (preferred);
-        g_simple_async_result_set_error (result,
+        g_simple_async_result_set_error (ctx->result,
                                          MM_CORE_ERROR,
                                          MM_CORE_ERROR_FAILED,
                                          "Requested mode (allowed: '%s', preferred: '%s') not "
@@ -290,85 +419,82 @@ set_allowed_modes (MMIfaceModem *_self,
         g_free (allowed_str);
         g_free (preferred_str);
 
-        g_simple_async_result_complete_in_idle (result);
-        g_object_unref (result);
+        set_current_modes_context_complete_and_free (ctx);
         return;
     }
 
-    /* Cache the value for next power-ups */
-    self->priv->network_mode = mbm_mode;
-
-    command = g_strdup_printf ("+CFUN=%d", mbm_mode);
+    command = g_strdup_printf ("+CFUN=%d", ctx->mbm_mode);
     mm_base_modem_at_command (
         MM_BASE_MODEM (self),
         command,
         3,
         FALSE,
         (GAsyncReadyCallback)allowed_mode_update_ready,
-        result);
+        ctx);
     g_free (command);
 }
 
 /*****************************************************************************/
-/* Initializing the modem (Modem interface) */
+/* Initializing the modem (during first enabling) */
 
 typedef struct {
     GSimpleAsyncResult *result;
     MMBroadbandModemMbm *self;
-} ModemInitContext;
+} EnablingModemInitContext;
 
 static void
-modem_init_context_complete_and_free (ModemInitContext *ctx)
+enabling_modem_init_context_complete_and_free (EnablingModemInitContext *ctx)
 {
     g_simple_async_result_complete (ctx->result);
     g_object_unref (ctx->result);
     g_object_unref (ctx->self);
-    g_slice_free (ModemInitContext, ctx);
+    g_slice_free (EnablingModemInitContext, ctx);
 }
 
 static gboolean
-modem_init_finish (MMIfaceModem *self,
-                   GAsyncResult *res,
-                   GError **error)
+enabling_modem_init_finish (MMBroadbandModem *self,
+                            GAsyncResult *res,
+                            GError **error)
 {
-    /* Ignore errors */
-    mm_base_modem_at_sequence_finish (MM_BASE_MODEM (self), res, NULL, NULL);
-    return TRUE;
+    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
 }
 
 static void
-init_sequence_ready (MMBaseModem *self,
-                     GAsyncResult *res,
-                     ModemInitContext *ctx)
+enabling_init_sequence_ready (MMBaseModem *self,
+                              GAsyncResult *res,
+                              EnablingModemInitContext *ctx)
 {
-    mm_base_modem_at_sequence_finish (self, res, NULL, NULL);
+    /* Ignore errors */
+    mm_base_modem_at_sequence_full_finish (self, res, NULL, NULL);
     g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-    modem_init_context_complete_and_free (ctx);
+    enabling_modem_init_context_complete_and_free (ctx);
 }
 
-static const MMBaseModemAtCommand modem_init_sequence[] = {
+static const MMBaseModemAtCommand enabling_modem_init_sequence[] = {
     /* Init command */
-    { "&F E0 V1 X4 &C1 +CMEE=1", 3, FALSE, NULL },
+    { "&F", 3, FALSE, NULL },
     /* Ensure disconnected */
     { "*ENAP=0", 3, FALSE, NULL },
     { NULL }
 };
 
 static void
-run_init_sequence (ModemInitContext *ctx)
+run_enabling_init_sequence (EnablingModemInitContext *ctx)
 {
-    mm_base_modem_at_sequence (MM_BASE_MODEM (ctx->self),
-                               modem_init_sequence,
-                               NULL,  /* response_processor_context */
-                               NULL,  /* response_processor_context_free */
-                               (GAsyncReadyCallback)init_sequence_ready,
-                               ctx);
+    mm_base_modem_at_sequence_full (MM_BASE_MODEM (ctx->self),
+                                    mm_base_modem_peek_port_primary (MM_BASE_MODEM (ctx->self)),
+                                    enabling_modem_init_sequence,
+                                    NULL,  /* response_processor_context */
+                                    NULL,  /* response_processor_context_free */
+                                    NULL, /* cancellable */
+                                    (GAsyncReadyCallback)enabling_init_sequence_ready,
+                                    ctx);
 }
 
 static void
 emrdy_ready (MMBaseModem *self,
              GAsyncResult *res,
-             ModemInitContext *ctx)
+             EnablingModemInitContext *ctx)
 {
     GError *error = NULL;
 
@@ -391,26 +517,26 @@ emrdy_ready (MMBaseModem *self,
         g_error_free (error);
     }
 
-    run_init_sequence (ctx);
+    run_enabling_init_sequence (ctx);
 }
 
 static void
-modem_init (MMIfaceModem *self,
-            GAsyncReadyCallback callback,
-            gpointer user_data)
+enabling_modem_init (MMBroadbandModem *self,
+                     GAsyncReadyCallback callback,
+                     gpointer user_data)
 {
-    ModemInitContext *ctx;
+    EnablingModemInitContext *ctx;
 
-    ctx = g_slice_new0 (ModemInitContext);
+    ctx = g_slice_new0 (EnablingModemInitContext);
     ctx->result = g_simple_async_result_new (G_OBJECT (self),
                                              callback,
                                              user_data,
-                                             modem_init);
+                                             enabling_modem_init);
     ctx->self = g_object_ref (self);
 
     /* Modem is ready?, no need to check EMRDY */
     if (ctx->self->priv->have_emrdy) {
-        run_init_sequence (ctx);
+        run_enabling_init_sequence (ctx);
         return;
     }
 
@@ -422,6 +548,32 @@ modem_init (MMIfaceModem *self,
                               ctx);
 }
 
+/*****************************************************************************/
+/* Modem power down (Modem interface) */
+
+static gboolean
+modem_power_down_finish (MMIfaceModem *self,
+                         GAsyncResult *res,
+                         GError **error)
+{
+    return !!mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, error);
+}
+
+static void
+modem_power_down (MMIfaceModem *self,
+                  GAsyncReadyCallback callback,
+                  gpointer user_data)
+{
+    /* Use AT+CFUN=4 for power down. It will stop the RF (IMSI detach), and
+     * keeps access to the SIM */
+    mm_base_modem_at_command (MM_BASE_MODEM (self),
+                              "+CFUN=4",
+                              3,
+                              FALSE,
+                              callback,
+                              user_data);
+}
+
 /*****************************************************************************/
 /* Powering up the modem (Modem interface) */
 
@@ -443,7 +595,11 @@ modem_power_up (MMIfaceModem *_self,
     MMBroadbandModemMbm *self = MM_BROADBAND_MODEM_MBM (_self);
     gchar *command;
 
-    command = g_strdup_printf ("+CFUN=%u", self->priv->network_mode);
+    g_assert (self->priv->mbm_mode == MBM_NETWORK_MODE_ANY ||
+              self->priv->mbm_mode == MBM_NETWORK_MODE_2G  ||
+              self->priv->mbm_mode == MBM_NETWORK_MODE_3G);
+
+    command = g_strdup_printf ("+CFUN=%u", self->priv->mbm_mode);
     mm_base_modem_at_command (MM_BASE_MODEM (self),
                               command,
                               5,
@@ -453,6 +609,59 @@ modem_power_up (MMIfaceModem *_self,
     g_free (command);
 }
 
+/*****************************************************************************/
+/* Power state loading (Modem interface) */
+
+static MMModemPowerState
+load_power_state_finish (MMIfaceModem *self,
+                         GAsyncResult *res,
+                         GError **error)
+{
+    const gchar *response;
+    guint a;
+
+    response = mm_base_modem_at_command_finish (MM_BASE_MODEM (self), res, error);
+    if (!response)
+        return FALSE;
+
+    if (mm_get_uint_from_str (mm_strip_tag (response, "+CFUN:"), &a)) {
+        switch (a) {
+        case MBM_NETWORK_MODE_OFFLINE:
+            return MM_MODEM_POWER_STATE_OFF;
+
+        case MBM_NETWORK_MODE_LOW_POWER:
+            return MM_MODEM_POWER_STATE_LOW;
+
+        case MBM_NETWORK_MODE_ANY:
+        case MBM_NETWORK_MODE_2G:
+        case MBM_NETWORK_MODE_3G:
+            return MM_MODEM_POWER_STATE_ON;
+        default:
+            break;
+        }
+    }
+
+    g_set_error (error,
+                 MM_CORE_ERROR,
+                 MM_CORE_ERROR_FAILED,
+                 "Couldn't parse +CFUN response: '%s'",
+                 response);
+    return MM_MODEM_POWER_STATE_UNKNOWN;
+}
+
+static void
+load_power_state (MMIfaceModem *self,
+                  GAsyncReadyCallback callback,
+                  gpointer user_data)
+{
+    mm_base_modem_at_command (MM_BASE_MODEM (self),
+                              "+CFUN?",
+                              3,
+                              FALSE,
+                              callback,
+                              user_data);
+}
+
 /*****************************************************************************/
 /* Reset (Modem interface) */
 
@@ -578,7 +787,7 @@ load_unlock_retries (MMIfaceModem *self,
     mm_dbg ("loading unlock retries (mbm)...");
     mm_base_modem_at_command (MM_BASE_MODEM (self),
                               "*EPIN?",
-                              3,
+                              10,
                               FALSE,
                               callback,
                               user_data);
@@ -588,15 +797,14 @@ load_unlock_retries (MMIfaceModem *self,
 /* Setup/Cleanup unsolicited events (3GPP interface) */
 
 typedef struct {
-    MMBroadbandBearerMbmConnectionStatus status;
+    MMBearerConnectionStatus status;
 } BearerListReportStatusForeachContext;
 
 static void
 bearer_list_report_status_foreach (MMBearer *bearer,
                                    BearerListReportStatusForeachContext *ctx)
 {
-    mm_broadband_bearer_mbm_report_connection_status (MM_BROADBAND_BEARER_MBM (bearer),
-                                                      ctx->status);
+    mm_bearer_report_connection_status (bearer, ctx->status);
 }
 
 static void
@@ -611,16 +819,16 @@ e2nap_received (MMAtSerialPort *port,
     if (!mm_get_uint_from_match_info (info, 1, &state))
         return;
 
-    ctx.status = MM_BROADBAND_BEARER_MBM_CONNECTION_STATUS_UNKNOWN;
+    ctx.status = MM_BEARER_CONNECTION_STATUS_UNKNOWN;
 
     switch (state) {
     case MBM_E2NAP_DISCONNECTED:
         mm_dbg ("disconnected");
-        ctx.status = MM_BROADBAND_BEARER_MBM_CONNECTION_STATUS_DISCONNECTED;
+        ctx.status = MM_BEARER_CONNECTION_STATUS_DISCONNECTED;
         break;
     case MBM_E2NAP_CONNECTED:
         mm_dbg ("connected");
-        ctx.status = MM_BROADBAND_BEARER_MBM_CONNECTION_STATUS_CONNECTED;
+        ctx.status = MM_BEARER_CONNECTION_STATUS_CONNECTED;
         break;
     case MBM_E2NAP_CONNECTING:
         mm_dbg ("connecting");
@@ -631,7 +839,7 @@ e2nap_received (MMAtSerialPort *port,
     }
 
     /* If unknown status, don't try to report anything */
-    if (ctx.status == MM_BROADBAND_BEARER_MBM_CONNECTION_STATUS_UNKNOWN)
+    if (ctx.status == MM_BEARER_CONNECTION_STATUS_UNKNOWN)
         return;
 
     /* If empty bearer list, nothing else to do */
@@ -677,6 +885,9 @@ erinfo_received (MMAtSerialPort *port,
         case 2:
             act = MM_MODEM_ACCESS_TECHNOLOGY_HSDPA;
             break;
+        case 3:
+            act = MM_MODEM_ACCESS_TECHNOLOGY_HSPA;
+            break;
         default:
             break;
         }
@@ -1018,6 +1229,11 @@ setup_ports (MMBroadbandModem *_self)
             self->priv->estksmenu_regex,
             NULL, NULL, NULL);
 
+        mm_at_serial_port_add_unsolicited_msg_handler (
+            ports[i],
+            self->priv->estksms_regex,
+            NULL, NULL, NULL);
+
         mm_at_serial_port_add_unsolicited_msg_handler (
             ports[i],
             self->priv->emwi_regex,
@@ -1054,8 +1270,6 @@ mm_broadband_modem_mbm_init (MMBroadbandModemMbm *self)
                                               MM_TYPE_BROADBAND_MODEM_MBM,
                                               MMBroadbandModemMbmPrivate);
 
-    self->priv->network_mode = MBM_NETWORK_MODE_ANY;
-
     /* Prepare regular expressions to setup */
     self->priv->e2nap_regex = g_regex_new ("\\r\\n\\*E2NAP: (\\d)\\r\\n",
                                            G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
@@ -1067,10 +1281,14 @@ mm_broadband_modem_mbm_init (MMBroadbandModemMbm *self)
                                            G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
     self->priv->estksmenu_regex = g_regex_new ("\\R\\*ESTKSMENU:.*\\R",
                                                G_REGEX_RAW | G_REGEX_OPTIMIZE | G_REGEX_MULTILINE | G_REGEX_NEWLINE_CRLF, G_REGEX_MATCH_NEWLINE_CRLF, NULL);
+    self->priv->estksms_regex = g_regex_new ("\\r\\n\\*ESTKSMS:.*\\r\\n",
+                                             G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
     self->priv->emwi_regex = g_regex_new ("\\r\\n\\*EMWI: (\\d),(\\d).*\\r\\n",
                                           G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
     self->priv->erinfo_regex = g_regex_new ("\\r\\n\\*ERINFO:\\s*(\\d),(\\d),(\\d).*\\r\\n",
-                                            G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);;
+                                            G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+
+    self->priv->mbm_mode = MBM_NETWORK_MODE_ANY;
 }
 
 static void
@@ -1083,6 +1301,7 @@ finalize (GObject *object)
     g_regex_unref (self->priv->emrdy_regex);
     g_regex_unref (self->priv->pacsp_regex);
     g_regex_unref (self->priv->estksmenu_regex);
+    g_regex_unref (self->priv->estksms_regex);
     g_regex_unref (self->priv->emwi_regex);
     g_regex_unref (self->priv->erinfo_regex);
 
@@ -1092,24 +1311,32 @@ finalize (GObject *object)
 static void
 iface_modem_init (MMIfaceModem *iface)
 {
+    iface_modem_parent = g_type_interface_peek_parent (iface);
+
     iface->create_bearer = modem_create_bearer;
     iface->create_bearer_finish = modem_create_bearer_finish;
+    iface->create_sim = create_sim;
+    iface->create_sim_finish = create_sim_finish;
     iface->modem_after_sim_unlock = modem_after_sim_unlock;
     iface->modem_after_sim_unlock_finish = modem_after_sim_unlock_finish;
-    iface->load_allowed_modes = load_allowed_modes;
-    iface->load_allowed_modes_finish = load_allowed_modes_finish;
-    iface->set_allowed_modes = set_allowed_modes;
-    iface->set_allowed_modes_finish = set_allowed_modes_finish;
-    iface->modem_init = modem_init;
-    iface->modem_init_finish = modem_init_finish;
-    iface->modem_power_up = modem_power_up;
-    iface->modem_power_up_finish = modem_power_up_finish;
+    iface->load_supported_modes = load_supported_modes;
+    iface->load_supported_modes_finish = load_supported_modes_finish;
+    iface->load_current_modes = load_current_modes;
+    iface->load_current_modes_finish = load_current_modes_finish;
+    iface->set_current_modes = set_current_modes;
+    iface->set_current_modes_finish = set_current_modes_finish;
     iface->reset = reset;
     iface->reset_finish = reset_finish;
     iface->factory_reset = factory_reset;
     iface->factory_reset_finish = factory_reset_finish;
     iface->load_unlock_retries = load_unlock_retries;
     iface->load_unlock_retries_finish = load_unlock_retries_finish;
+    iface->load_power_state = load_power_state;
+    iface->load_power_state_finish = load_power_state_finish;
+    iface->modem_power_up = modem_power_up;
+    iface->modem_power_up_finish = modem_power_up_finish;
+    iface->modem_power_down = modem_power_down;
+    iface->modem_power_down_finish = modem_power_down_finish;
 }
 
 static void
@@ -1138,4 +1365,6 @@ mm_broadband_modem_mbm_class_init (MMBroadbandModemMbmClass *klass)
 
     object_class->finalize = finalize;
     broadband_modem_class->setup_ports = setup_ports;
+    broadband_modem_class->enabling_modem_init = enabling_modem_init;
+    broadband_modem_class->enabling_modem_init_finish = enabling_modem_init_finish;
 }
