@@ -32,6 +32,7 @@
 #include "mm-broadband-bearer-hso.h"
 #include "mm-log.h"
 #include "mm-modem-helpers.h"
+#include "mm-daemon-enums-types.h"
 
 G_DEFINE_TYPE (MMBroadbandBearerHso, mm_broadband_bearer_hso, MM_TYPE_BROADBAND_BEARER);
 
@@ -143,6 +144,7 @@ ip_config_ready (MMBaseModem *modem,
             ip_config = mm_bearer_ip_config_new ();
             mm_bearer_ip_config_set_method (ip_config, MM_BEARER_IP_METHOD_STATIC);
             mm_bearer_ip_config_set_address (ip_config,  items[i]);
+            mm_bearer_ip_config_set_prefix (ip_config, 32);
         } else if (i == 3 || i == 4) { /* DNS entries */
             guint32 tmp;
 
@@ -313,44 +315,56 @@ connect_reset (Dial3gppContext *ctx)
     g_free (command);
 }
 
-void
-mm_broadband_bearer_hso_report_connection_status (MMBroadbandBearerHso *self,
-                                                  MMBroadbandBearerHsoConnectionStatus status)
+static void
+report_connection_status (MMBearer *bearer,
+                          MMBearerConnectionStatus status)
 {
+    MMBroadbandBearerHso *self = MM_BROADBAND_BEARER_HSO (bearer);
     Dial3gppContext *ctx;
 
+    g_assert (status == MM_BEARER_CONNECTION_STATUS_CONNECTED ||
+              status == MM_BEARER_CONNECTION_STATUS_CONNECTION_FAILED ||
+              status == MM_BEARER_CONNECTION_STATUS_DISCONNECTED);
+
     /* Recover context (if any) and remove both cancellation and timeout (if any)*/
     ctx = self->priv->connect_pending;
     self->priv->connect_pending = NULL;
 
+    /* Connection status reported but no connection attempt? */
+    if (!ctx) {
+        g_assert (self->priv->connect_pending_id == 0);
+
+        mm_dbg ("Received spontaneous _OWANCALL (%s)",
+                mm_bearer_connection_status_get_string (status));
+
+        if (status == MM_BEARER_CONNECTION_STATUS_DISCONNECTED) {
+            /* If no connection attempt on-going, make sure we mark ourselves as
+             * disconnected */
+            MM_BEARER_CLASS (mm_broadband_bearer_hso_parent_class)->report_connection_status (
+                bearer,
+                status);
+        }
+        return;
+    }
+
     if (self->priv->connect_pending_id) {
         g_source_remove (self->priv->connect_pending_id);
         self->priv->connect_pending_id = 0;
     }
 
-    if (ctx && self->priv->connect_cancellable_id) {
+    if (self->priv->connect_cancellable_id) {
         g_cancellable_disconnect (ctx->cancellable,
                                   self->priv->connect_cancellable_id);
         self->priv->connect_cancellable_id = 0;
     }
 
-    if (ctx && self->priv->connect_port_closed_id) {
+    if (self->priv->connect_port_closed_id) {
         g_signal_handler_disconnect (ctx->primary, self->priv->connect_port_closed_id);
         self->priv->connect_port_closed_id = 0;
     }
 
-    switch (status) {
-    case MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_UNKNOWN:
-        break;
-
-    case MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_CONNECTED:
-        if (!ctx)
-            /* We may get this if the timeout for the connection attempt is
-             * reached before the unsolicited response. We should probably
-             * keep the CID around to request explicit disconnection in this
-             * case. */
-            break;
-
+    /* Reporting connected */
+    if (status == MM_BEARER_CONNECTION_STATUS_CONNECTED) {
         /* If we wanted to get cancelled before, do it now */
         if (ctx->saved_error) {
             /* Keep error */
@@ -366,52 +380,23 @@ mm_broadband_bearer_hso_report_connection_status (MMBroadbandBearerHso *self,
                                                    (GDestroyNotify)g_object_unref);
         dial_3gpp_context_complete_and_free (ctx);
         return;
+    }
 
-    case MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_CONNECTION_FAILED:
-        if (!ctx)
-            break;
-
-        /* If we wanted to get cancelled before and now we couldn't connect,
-         * use the cancelled error and return */
-        if (ctx->saved_error) {
-            g_simple_async_result_take_error (ctx->result, ctx->saved_error);
-            ctx->saved_error = NULL;
-            dial_3gpp_context_complete_and_free (ctx);
-            return;
-        }
-
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "Call setup failed");
+    /* If we wanted to get cancelled before and now we couldn't connect,
+     * use the cancelled error and return */
+    if (ctx->saved_error) {
+        g_simple_async_result_take_error (ctx->result, ctx->saved_error);
+        ctx->saved_error = NULL;
         dial_3gpp_context_complete_and_free (ctx);
         return;
-
-    case MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_DISCONNECTED:
-        if (ctx) {
-            /* If we wanted to get cancelled before and now we couldn't connect,
-             * use the cancelled error and return */
-            if (ctx->saved_error) {
-                g_simple_async_result_take_error (ctx->result, ctx->saved_error);
-                ctx->saved_error = NULL;
-                dial_3gpp_context_complete_and_free (ctx);
-                return;
-            }
-
-            g_simple_async_result_set_error (ctx->result,
-                                             MM_CORE_ERROR,
-                                             MM_CORE_ERROR_FAILED,
-                                             "Call setup failed");
-            dial_3gpp_context_complete_and_free (ctx);
-            return;
-        }
-
-        /* Just ensure we mark ourselves as being disconnected... */
-        mm_bearer_report_disconnection (MM_BEARER (self));
-        return;
     }
 
-    g_warn_if_reached ();
+    /* Received CONNECTION_FAILED or DISCONNECTED during a connection attempt? */
+    g_simple_async_result_set_error (ctx->result,
+                                     MM_CORE_ERROR,
+                                     MM_CORE_ERROR_FAILED,
+                                     "Call setup failed");
+    dial_3gpp_context_complete_and_free (ctx);
 }
 
 static gboolean
@@ -481,9 +466,8 @@ forced_close_cb (MMSerialPort *port,
                  MMBroadbandBearerHso *self)
 {
     /* Just treat the forced close event as any other unsolicited message */
-    mm_broadband_bearer_hso_report_connection_status (
-        self,
-        MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_CONNECTION_FAILED);
+    mm_bearer_report_connection_status (MM_BEARER (self),
+                                        MM_BEARER_CONNECTION_STATUS_CONNECTION_FAILED);
 }
 
 static void
@@ -853,10 +837,12 @@ static void
 mm_broadband_bearer_hso_class_init (MMBroadbandBearerHsoClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    MMBearerClass *bearer_class = MM_BEARER_CLASS (klass);
     MMBroadbandBearerClass *broadband_bearer_class = MM_BROADBAND_BEARER_CLASS (klass);
 
     g_type_class_add_private (object_class, sizeof (MMBroadbandBearerHsoPrivate));
 
+    bearer_class->report_connection_status = report_connection_status;
     broadband_bearer_class->dial_3gpp = dial_3gpp;
     broadband_bearer_class->dial_3gpp_finish = dial_3gpp_finish;
     broadband_bearer_class->get_ip_config_3gpp = get_ip_config_3gpp;
