@@ -65,11 +65,32 @@ modem_create_bearer_finish (MMIfaceModem *self,
     MMBearer *bearer;
 
     bearer = g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res));
-    mm_dbg ("New HSO bearer created at DBus path '%s'", mm_bearer_get_path (bearer));
+    mm_dbg ("New %s bearer created at DBus path '%s'",
+            MM_IS_BROADBAND_BEARER_HSO (bearer) ? "HSO" : "Generic",
+            mm_bearer_get_path (bearer));
 
     return g_object_ref (bearer);
 }
 
+static void
+broadband_bearer_new_ready (GObject *source,
+                            GAsyncResult *res,
+                            GSimpleAsyncResult *simple)
+{
+    MMBearer *bearer = NULL;
+    GError *error = NULL;
+
+    bearer = mm_broadband_bearer_new_finish (res, &error);
+    if (!bearer)
+        g_simple_async_result_take_error (simple, error);
+    else
+        g_simple_async_result_set_op_res_gpointer (simple,
+                                                   bearer,
+                                                   (GDestroyNotify)g_object_unref);
+    g_simple_async_result_complete (simple);
+    g_object_unref (simple);
+}
+
 static void
 broadband_bearer_hso_new_ready (GObject *source,
                                 GAsyncResult *res,
@@ -102,6 +123,17 @@ modem_create_bearer (MMIfaceModem *self,
                                         user_data,
                                         modem_create_bearer);
 
+    if (mm_bearer_properties_get_ip_type (properties) &
+        (MM_BEARER_IP_FAMILY_IPV6 | MM_BEARER_IP_FAMILY_IPV4V6)) {
+        mm_dbg ("Creating generic bearer (IPv6 requested)...");
+        mm_broadband_bearer_new (MM_BROADBAND_MODEM (self),
+                                 properties,
+                                 NULL, /* cancellable */
+                                 (GAsyncReadyCallback)broadband_bearer_new_ready,
+                                 result);
+        return;
+    }
+
     mm_dbg ("Creating HSO bearer...");
     mm_broadband_bearer_hso_new (MM_BROADBAND_MODEM_HSO (self),
                                  properties,
@@ -184,7 +216,7 @@ load_unlock_retries (MMIfaceModem *self,
 
 typedef struct {
     guint cid;
-    MMBroadbandBearerHsoConnectionStatus status;
+    MMBearerConnectionStatus status;
 } BearerListReportStatusForeachContext;
 
 static void
@@ -194,8 +226,7 @@ bearer_list_report_status_foreach (MMBearer *bearer,
     if (mm_broadband_bearer_get_3gpp_cid (MM_BROADBAND_BEARER (bearer)) != ctx->cid)
         return;
 
-    mm_broadband_bearer_hso_report_connection_status (MM_BROADBAND_BEARER_HSO (bearer),
-                                                      ctx->status);
+    mm_bearer_report_connection_status (MM_BEARER (bearer), ctx->status);
 }
 
 static void
@@ -215,24 +246,24 @@ hso_connection_status_changed (MMAtSerialPort *port,
 
     /* Setup context */
     ctx.cid = cid;
-    ctx.status = MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_UNKNOWN;
+    ctx.status = MM_BEARER_CONNECTION_STATUS_UNKNOWN;
 
     switch (status) {
     case 1:
-        ctx.status = MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_CONNECTED;
+        ctx.status = MM_BEARER_CONNECTION_STATUS_CONNECTED;
         break;
     case 3:
-        ctx.status = MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_CONNECTION_FAILED;
+        ctx.status = MM_BEARER_CONNECTION_STATUS_CONNECTION_FAILED;
         break;
     case 0:
-        ctx.status = MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_DISCONNECTED;
+        ctx.status = MM_BEARER_CONNECTION_STATUS_DISCONNECTED;
         break;
     default:
         break;
     }
 
     /* If unknown status, don't try to report anything */
-    if (ctx.status == MM_BROADBAND_BEARER_HSO_CONNECTION_STATUS_UNKNOWN)
+    if (ctx.status == MM_BEARER_CONNECTION_STATUS_UNKNOWN)
         return;
 
     /* If empty bearer list, nothing else to do */
