@@ -31,6 +31,9 @@ G_DEFINE_TYPE (MMAtSerialPort, mm_at_serial_port, MM_TYPE_SERIAL_PORT)
 enum {
     PROP_0,
     PROP_REMOVE_ECHO,
+    PROP_INIT_SEQUENCE_ENABLED,
+    PROP_INIT_SEQUENCE,
+    PROP_SEND_LF,
     LAST_PROP
 };
 
@@ -44,7 +47,11 @@ typedef struct {
 
     MMAtPortFlag flags;
 
+    /* Properties */
     gboolean remove_echo;
+    guint init_sequence_enabled;
+    gchar **init_sequence;
+    gboolean send_lf;
 } MMAtSerialPortPrivate;
 
 /*****************************************************************************/
@@ -169,6 +176,7 @@ handle_response (MMSerialPort *port,
 typedef struct {
     GRegex *regex;
     MMAtSerialUnsolicitedMsgFn callback;
+    gboolean enable;
     gpointer user_data;
     GDestroyNotify notify;
 } MMAtUnsolicitedMsgHandler;
@@ -212,10 +220,34 @@ mm_at_serial_port_add_unsolicited_msg_handler (MMAtSerialPort *self,
     }
 
     handler->callback = callback;
+    handler->enable = TRUE;
     handler->user_data = user_data;
     handler->notify = notify;
 }
 
+void
+mm_at_serial_port_enable_unsolicited_msg_handler (MMAtSerialPort *self,
+                                                  GRegex *regex,
+                                                  gboolean enable)
+{
+    GSList *existing;
+    MMAtUnsolicitedMsgHandler *handler;
+    MMAtSerialPortPrivate *priv;
+
+    g_return_if_fail (MM_IS_AT_SERIAL_PORT (self));
+    g_return_if_fail (regex != NULL);
+
+    priv = MM_AT_SERIAL_PORT_GET_PRIVATE (self);
+
+    existing = g_slist_find_custom (priv->unsolicited_msg_handlers,
+                                    regex,
+                                    (GCompareFunc)unsolicited_msg_handler_cmp);
+    if (existing) {
+        handler = existing->data;
+        handler->enable = enable;
+    }
+}
+
 static gboolean
 remove_eval_cb (const GMatchInfo *match_info,
                 GString *result,
@@ -247,6 +279,9 @@ parse_unsolicited (MMSerialPort *port, GByteArray *response)
         GMatchInfo *match_info;
         gboolean matches;
 
+        if (!handler->enable)
+            continue;
+
         matches = g_regex_match_full (handler->regex,
                                       (const char *) response->data,
                                       response->len,
@@ -281,7 +316,7 @@ parse_unsolicited (MMSerialPort *port, GByteArray *response)
 /*****************************************************************************/
 
 static GByteArray *
-at_command_to_byte_array (const char *command, gboolean is_raw)
+at_command_to_byte_array (const char *command, gboolean is_raw, gboolean send_lf)
 {
     GByteArray *buf;
     int cmdlen;
@@ -289,7 +324,7 @@ at_command_to_byte_array (const char *command, gboolean is_raw)
     g_return_val_if_fail (command != NULL, NULL);
 
     cmdlen = strlen (command);
-    buf = g_byte_array_sized_new (cmdlen + 3);
+    buf = g_byte_array_sized_new (cmdlen + 4);
 
     if (!is_raw) {
         /* Make sure there's an AT in the front */
@@ -301,8 +336,15 @@ at_command_to_byte_array (const char *command, gboolean is_raw)
 
     if (!is_raw) {
         /* Make sure there's a trailing carriage return */
-        if (command[cmdlen - 1] != '\r')
-            g_byte_array_append (buf, (const guint8 *) "\r", 1);
+        if ((cmdlen == 0) ||
+            (command[cmdlen - 1] != '\r' && (cmdlen == 1 || command[cmdlen - 2] != '\r')))
+             g_byte_array_append (buf, (const guint8 *) "\r", 1);
+        if (send_lf) {
+            /* Make sure there's a trailing line-feed */
+            if ((cmdlen == 0) ||
+                (command[cmdlen - 1] != '\n' && (cmdlen == 1 || command[cmdlen - 2] != '\n')))
+                 g_byte_array_append (buf, (const guint8 *) "\n", 1);
+        }
     }
 
     return buf;
@@ -318,12 +360,13 @@ mm_at_serial_port_queue_command (MMAtSerialPort *self,
                                  gpointer user_data)
 {
     GByteArray *buf;
+    MMAtSerialPortPrivate *priv = MM_AT_SERIAL_PORT_GET_PRIVATE (self);
 
     g_return_if_fail (self != NULL);
     g_return_if_fail (MM_IS_AT_SERIAL_PORT (self));
     g_return_if_fail (command != NULL);
 
-    buf = at_command_to_byte_array (command, is_raw);
+    buf = at_command_to_byte_array (command, is_raw, priv->send_lf);
     g_return_if_fail (buf != NULL);
 
     mm_serial_port_queue_command (MM_SERIAL_PORT (self),
@@ -345,12 +388,13 @@ mm_at_serial_port_queue_command_cached (MMAtSerialPort *self,
                                         gpointer user_data)
 {
     GByteArray *buf;
+    MMAtSerialPortPrivate *priv = MM_AT_SERIAL_PORT_GET_PRIVATE (self);
 
     g_return_if_fail (self != NULL);
     g_return_if_fail (MM_IS_AT_SERIAL_PORT (self));
     g_return_if_fail (command != NULL);
 
-    buf = at_command_to_byte_array (command, is_raw);
+    buf = at_command_to_byte_array (command, is_raw, priv->send_lf);
     g_return_if_fail (buf != NULL);
 
     mm_serial_port_queue_command_cached (MM_SERIAL_PORT (self),
@@ -417,6 +461,40 @@ mm_at_serial_port_get_flags (MMAtSerialPort *self)
 
 /*****************************************************************************/
 
+void
+mm_at_serial_port_run_init_sequence (MMAtSerialPort *self)
+{
+    MMAtSerialPortPrivate *priv = MM_AT_SERIAL_PORT_GET_PRIVATE (self);
+    guint i;
+
+    if (!priv->init_sequence)
+        return;
+
+    mm_dbg ("(%s): running init sequence...", mm_port_get_device (MM_PORT (self)));
+
+    /* Just queue the init commands, don't wait for reply */
+    for (i = 0; priv->init_sequence[i]; i++) {
+        mm_at_serial_port_queue_command (self,
+                                         priv->init_sequence[i],
+                                         3,
+                                         FALSE,
+                                         NULL,
+                                         NULL,
+                                         NULL);
+    }
+}
+
+static void
+config (MMSerialPort *self)
+{
+    MMAtSerialPortPrivate *priv = MM_AT_SERIAL_PORT_GET_PRIVATE (self);
+
+    if (priv->init_sequence_enabled)
+        mm_at_serial_port_run_init_sequence (MM_AT_SERIAL_PORT (self));
+}
+
+/*****************************************************************************/
+
 MMAtSerialPort *
 mm_at_serial_port_new (const char *name)
 {
@@ -434,6 +512,11 @@ mm_at_serial_port_init (MMAtSerialPort *self)
 
     /* By default, remove echo */
     priv->remove_echo = TRUE;
+    /* By default, run init sequence during first port opening */
+    priv->init_sequence_enabled = TRUE;
+
+    /* By default, don't send line feed */
+    priv->send_lf = FALSE;
 }
 
 static void
@@ -446,6 +529,16 @@ set_property (GObject *object, guint prop_id,
     case PROP_REMOVE_ECHO:
         priv->remove_echo = g_value_get_boolean (value);
         break;
+    case PROP_INIT_SEQUENCE_ENABLED:
+        priv->init_sequence_enabled = g_value_get_boolean (value);
+        break;
+    case PROP_INIT_SEQUENCE:
+        g_strfreev (priv->init_sequence);
+        priv->init_sequence = g_value_dup_boxed (value);
+        break;
+    case PROP_SEND_LF:
+        priv->send_lf = g_value_get_boolean (value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -462,6 +555,15 @@ get_property (GObject *object, guint prop_id,
     case PROP_REMOVE_ECHO:
         g_value_set_boolean (value, priv->remove_echo);
         break;
+    case PROP_INIT_SEQUENCE_ENABLED:
+        g_value_set_boolean (value, priv->init_sequence_enabled);
+        break;
+    case PROP_INIT_SEQUENCE:
+        g_value_set_boxed (value, priv->init_sequence);
+        break;
+    case PROP_SEND_LF:
+        g_value_set_boolean (value, priv->send_lf);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -489,6 +591,8 @@ finalize (GObject *object)
     if (priv->response_parser_notify)
         priv->response_parser_notify (priv->response_parser_user_data);
 
+    g_strfreev (priv->init_sequence);
+
     G_OBJECT_CLASS (mm_at_serial_port_parent_class)->finalize (object);
 }
 
@@ -509,6 +613,7 @@ mm_at_serial_port_class_init (MMAtSerialPortClass *klass)
     port_class->parse_response = parse_response;
     port_class->handle_response = handle_response;
     port_class->debug_log = debug_log;
+    port_class->config = config;
 
     g_object_class_install_property
         (object_class, PROP_REMOVE_ECHO,
@@ -517,4 +622,28 @@ mm_at_serial_port_class_init (MMAtSerialPortClass *klass)
                                "Built-in echo removal should be applied",
                                TRUE,
                                G_PARAM_READWRITE));
+
+    g_object_class_install_property
+        (object_class, PROP_INIT_SEQUENCE_ENABLED,
+         g_param_spec_boolean (MM_AT_SERIAL_PORT_INIT_SEQUENCE_ENABLED,
+                               "Init sequence enabled",
+                               "Whether the initialization sequence should be run",
+                               TRUE,
+                               G_PARAM_READWRITE));
+
+    g_object_class_install_property
+        (object_class, PROP_INIT_SEQUENCE,
+         g_param_spec_boxed (MM_AT_SERIAL_PORT_INIT_SEQUENCE,
+                             "Init sequence",
+                             "Initialization sequence",
+                             G_TYPE_STRV,
+                             G_PARAM_READWRITE));
+
+    g_object_class_install_property
+        (object_class, PROP_SEND_LF,
+         g_param_spec_boolean (MM_AT_SERIAL_PORT_SEND_LF,
+                               "Send LF",
+                               "Send line-feed at the end of each AT command sent",
+                               FALSE,
+                               G_PARAM_READWRITE));
 }
