@@ -955,7 +955,7 @@ mm_modem_capability_from_qmi_radio_technology_preference (QmiNasRadioTechnologyP
 QmiNasRadioTechnologyPreference
 mm_modem_capability_to_qmi_radio_technology_preference (MMModemCapability caps)
 {
-    QmiNasRatModePreference qmi = 0;
+    QmiNasRadioTechnologyPreference qmi = 0;
 
     /* It is not expected to have a modem supporting 3GPP and 3GPP2 at the same
      * time but not supporting SSP. */
@@ -1079,7 +1079,7 @@ mm_modem_mode_to_qmi_gsm_wcdma_acquisition_order_preference (MMModemMode mode)
     mm_dbg ("Unhandled modem mode: '%s'", str);
     g_free (str);
 
-    return MM_MODEM_MODE_NONE;
+    return QMI_NAS_GSM_WCDMA_ACQUISITION_ORDER_PREFERENCE_AUTOMATIC;
 }
 
 /*****************************************************************************/
@@ -1093,7 +1093,7 @@ mm_modem_3gpp_registration_state_from_qmi_registration_state (QmiNasAttachState
         return MM_MODEM_3GPP_REGISTRATION_STATE_UNKNOWN;
 
     if (attach_state == QMI_NAS_ATTACH_STATE_DETACHED)
-        return QMI_NAS_REGISTRATION_STATE_NOT_REGISTERED;
+        return MM_MODEM_3GPP_REGISTRATION_STATE_IDLE;
 
     /* attached */
 
@@ -1271,3 +1271,106 @@ mm_modem_capability_from_qmi_capabilities_context (MMQmiCapabilitiesContext *ctx
 
     return tmp;
 }
+
+/*****************************************************************************/
+
+MMOmaSessionType
+mm_oma_session_type_from_qmi_oma_session_type (QmiOmaSessionType qmi_session_type)
+{
+    switch (qmi_session_type) {
+    case QMI_OMA_SESSION_TYPE_CLIENT_INITIATED_DEVICE_CONFIGURE:
+        return MM_OMA_SESSION_TYPE_CLIENT_INITIATED_DEVICE_CONFIGURE;
+    case QMI_OMA_SESSION_TYPE_CLIENT_INITIATED_PRL_UPDATE:
+        return MM_OMA_SESSION_TYPE_CLIENT_INITIATED_PRL_UPDATE;
+    case QMI_OMA_SESSION_TYPE_CLIENT_INITIATED_HANDS_FREE_ACTIVATION:
+        return MM_OMA_SESSION_TYPE_CLIENT_INITIATED_HANDS_FREE_ACTIVATION;
+    case QMI_OMA_SESSION_TYPE_DEVICE_INITIATED_HANDS_FREE_ACTIVATION:
+        return MM_OMA_SESSION_TYPE_DEVICE_INITIATED_HANDS_FREE_ACTIVATION;
+    case QMI_OMA_SESSION_TYPE_NETWORK_INITIATED_PRL_UPDATE:
+        return MM_OMA_SESSION_TYPE_NETWORK_INITIATED_PRL_UPDATE;
+    case QMI_OMA_SESSION_TYPE_NETWORK_INITIATED_DEVICE_CONFIGURE:
+        return MM_OMA_SESSION_TYPE_NETWORK_INITIATED_DEVICE_CONFIGURE;
+    case QMI_OMA_SESSION_TYPE_DEVICE_INITIATED_PRL_UPDATE:
+        return MM_OMA_SESSION_TYPE_DEVICE_INITIATED_PRL_UPDATE;
+    default:
+        return MM_OMA_SESSION_TYPE_UNKNOWN;
+    }
+}
+
+QmiOmaSessionType
+mm_oma_session_type_to_qmi_oma_session_type (MMOmaSessionType mm_session_type)
+{
+    switch (mm_session_type) {
+    case MM_OMA_SESSION_TYPE_CLIENT_INITIATED_DEVICE_CONFIGURE:
+        return QMI_OMA_SESSION_TYPE_CLIENT_INITIATED_DEVICE_CONFIGURE;
+    case MM_OMA_SESSION_TYPE_CLIENT_INITIATED_PRL_UPDATE:
+        return QMI_OMA_SESSION_TYPE_CLIENT_INITIATED_PRL_UPDATE;
+    case MM_OMA_SESSION_TYPE_CLIENT_INITIATED_HANDS_FREE_ACTIVATION:
+        return QMI_OMA_SESSION_TYPE_CLIENT_INITIATED_HANDS_FREE_ACTIVATION;
+    case MM_OMA_SESSION_TYPE_DEVICE_INITIATED_HANDS_FREE_ACTIVATION:
+        return QMI_OMA_SESSION_TYPE_DEVICE_INITIATED_HANDS_FREE_ACTIVATION;
+    case MM_OMA_SESSION_TYPE_NETWORK_INITIATED_PRL_UPDATE:
+        return QMI_OMA_SESSION_TYPE_NETWORK_INITIATED_PRL_UPDATE;
+    case MM_OMA_SESSION_TYPE_NETWORK_INITIATED_DEVICE_CONFIGURE:
+        return QMI_OMA_SESSION_TYPE_NETWORK_INITIATED_DEVICE_CONFIGURE;
+    case MM_OMA_SESSION_TYPE_DEVICE_INITIATED_PRL_UPDATE:
+        return QMI_OMA_SESSION_TYPE_DEVICE_INITIATED_PRL_UPDATE;
+    default:
+        g_assert_not_reached ();
+    }
+}
+
+MMOmaSessionState
+mm_oma_session_state_from_qmi_oma_session_state (QmiOmaSessionState qmi_session_state)
+{
+    /* Note: MM_OMA_SESSION_STATE_STARTED is not a state received from the modem */
+
+    switch (qmi_session_state) {
+    case QMI_OMA_SESSION_STATE_COMPLETE_INFORMATION_UPDATED:
+    case QMI_OMA_SESSION_STATE_COMPLETE_UPDATED_INFORMATION_UNAVAILABLE:
+        return MM_OMA_SESSION_STATE_COMPLETED;
+    case QMI_OMA_SESSION_STATE_FAILED:
+        return MM_OMA_SESSION_STATE_FAILED;
+    case QMI_OMA_SESSION_STATE_RETRYING:
+        return MM_OMA_SESSION_STATE_RETRYING;
+    case QMI_OMA_SESSION_STATE_CONNECTING:
+        return MM_OMA_SESSION_STATE_CONNECTING;
+    case QMI_OMA_SESSION_STATE_CONNECTED:
+        return MM_OMA_SESSION_STATE_CONNECTED;
+    case QMI_OMA_SESSION_STATE_AUTHENTICATED:
+        return MM_OMA_SESSION_STATE_AUTHENTICATED;
+    case QMI_OMA_SESSION_STATE_MDN_DOWNLOADED:
+        return MM_OMA_SESSION_STATE_MDN_DOWNLOADED;
+    case QMI_OMA_SESSION_STATE_MSID_DOWNLOADED:
+        return MM_OMA_SESSION_STATE_MSID_DOWNLOADED;
+    case QMI_OMA_SESSION_STATE_PRL_DOWNLOADED:
+        return MM_OMA_SESSION_STATE_PRL_DOWNLOADED;
+    case QMI_OMA_SESSION_STATE_MIP_PROFILE_DOWNLOADED:
+        return MM_OMA_SESSION_STATE_MIP_PROFILE_DOWNLOADED;
+    default:
+        return MM_OMA_SESSION_STATE_UNKNOWN;
+    }
+}
+
+/*****************************************************************************/
+
+MMOmaSessionStateFailedReason
+mm_oma_session_state_failed_reason_from_qmi_oma_session_failed_reason (QmiOmaSessionFailedReason qmi_session_failed_reason)
+{
+    switch (qmi_session_failed_reason) {
+    case QMI_OMA_SESSION_FAILED_REASON_UNKNOWN:
+        return MM_OMA_SESSION_STATE_FAILED_REASON_UNKNOWN;
+    case QMI_OMA_SESSION_FAILED_REASON_NETWORK_UNAVAILABLE:
+        return MM_OMA_SESSION_STATE_FAILED_REASON_NETWORK_UNAVAILABLE;
+    case QMI_OMA_SESSION_FAILED_REASON_SERVER_UNAVAILABLE:
+        return MM_OMA_SESSION_STATE_FAILED_REASON_SERVER_UNAVAILABLE;
+    case QMI_OMA_SESSION_FAILED_REASON_AUTHENTICATION_FAILED:
+        return MM_OMA_SESSION_STATE_FAILED_REASON_AUTHENTICATION_FAILED;
+    case QMI_OMA_SESSION_FAILED_REASON_MAX_RETRY_EXCEEDED:
+        return MM_OMA_SESSION_STATE_FAILED_REASON_MAX_RETRY_EXCEEDED;
+    case QMI_OMA_SESSION_FAILED_REASON_SESSION_CANCELLED:
+        return MM_OMA_SESSION_STATE_FAILED_REASON_SESSION_CANCELLED;
+    default:
+        return MM_OMA_SESSION_STATE_FAILED_REASON_UNKNOWN;
+    }
+}
