@@ -25,9 +25,12 @@
 #include <libmm-glib.h>
 
 #include "mm-modem-helpers-qmi.h"
+#include "mm-iface-modem.h"
 #include "mm-iface-modem-messaging.h"
 #include "mm-sms-qmi.h"
 #include "mm-base-modem.h"
+#include "mm-sms-part-3gpp.h"
+#include "mm-sms-part-cdma.h"
 #include "mm-log.h"
 
 G_DEFINE_TYPE (MMSmsQmi, mm_sms_qmi, MM_TYPE_SMS);
@@ -81,6 +84,33 @@ ensure_qmi_client (MMSmsQmi *self,
     return TRUE;
 }
 
+/*****************************************************************************/
+
+static gboolean
+check_sms_type_support (MMSmsQmi *self,
+                        MMBaseModem *modem,
+                        MMSmsPart *first_part,
+                        GError **error)
+{
+    if (MM_SMS_PART_IS_3GPP (first_part) && !mm_iface_modem_is_3gpp (MM_IFACE_MODEM (modem))) {
+        g_set_error (error,
+                     MM_CORE_ERROR,
+                     MM_CORE_ERROR_UNSUPPORTED,
+                     "Non-3GPP modem doesn't support 3GPP SMS");
+        return FALSE;
+    }
+
+    if (MM_SMS_PART_IS_CDMA (first_part) && !mm_iface_modem_is_cdma (MM_IFACE_MODEM (modem))) {
+        g_set_error (error,
+                     MM_CORE_ERROR,
+                     MM_CORE_ERROR_UNSUPPORTED,
+                     "Non-CDMA modem doesn't support CDMA SMS");
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 /*****************************************************************************/
 /* Store the SMS */
 
@@ -96,7 +126,7 @@ typedef struct {
 static void
 sms_store_context_complete_and_free (SmsStoreContext *ctx)
 {
-    g_simple_async_result_complete (ctx->result);
+    g_simple_async_result_complete_in_idle (ctx->result);
     g_object_unref (ctx->result);
     g_object_unref (ctx->client);
     g_object_unref (ctx->modem);
@@ -159,7 +189,7 @@ static void
 sms_store_next_part (SmsStoreContext *ctx)
 {
     QmiMessageWmsRawWriteInput *input;
-    guint8 *pdu;
+    guint8 *pdu = NULL;
     guint pdulen = 0;
     guint msgstart = 0;
     GArray *array;
@@ -173,10 +203,22 @@ sms_store_next_part (SmsStoreContext *ctx)
     }
 
     /* Get PDU */
-    pdu = mm_sms_part_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &msgstart, &error);
+    if (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data))
+        pdu = mm_sms_part_3gpp_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &msgstart, &error);
+    else if (MM_SMS_PART_IS_CDMA ((MMSmsPart *)ctx->current->data))
+        pdu = mm_sms_part_cdma_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &error);
+
     if (!pdu) {
-        /* 'error' should already be set */
-        g_simple_async_result_take_error (ctx->result, error);
+        if (error)
+            g_simple_async_result_take_error (ctx->result, error);
+        else
+            g_simple_async_result_set_error (ctx->result,
+                                             MM_CORE_ERROR,
+                                             MM_CORE_ERROR_FAILED,
+                                             "Unknown or unsupported PDU type in SMS part: %s",
+                                             mm_sms_pdu_type_get_string (
+                                                 mm_sms_part_get_pdu_type (
+                                                     (MMSmsPart *)ctx->current->data)));
         sms_store_context_complete_and_free (ctx);
         return;
     }
@@ -192,7 +234,9 @@ sms_store_next_part (SmsStoreContext *ctx)
     qmi_message_wms_raw_write_input_set_raw_message_data (
         input,
         mm_sms_storage_to_qmi_storage_type (ctx->storage),
-        QMI_WMS_MESSAGE_FORMAT_GSM_WCDMA_POINT_TO_POINT,
+        (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data) ?
+         QMI_WMS_MESSAGE_FORMAT_GSM_WCDMA_POINT_TO_POINT :
+         QMI_WMS_MESSAGE_FORMAT_CDMA),
         array,
         NULL);
     qmi_client_wms_raw_write (ctx->client,
@@ -213,6 +257,7 @@ sms_store (MMSms *self,
 {
     SmsStoreContext *ctx;
     QmiClient *client = NULL;
+    GError *error = NULL;
 
     /* Ensure WMS client */
     if (!ensure_qmi_client (MM_SMS_QMI (self),
@@ -234,6 +279,15 @@ sms_store (MMSms *self,
                   NULL);
 
     ctx->current = mm_sms_get_parts (self);
+
+    /* Check whether we support the given SMS type */
+    if (!check_sms_type_support (MM_SMS_QMI (self), ctx->modem, (MMSmsPart *)ctx->current->data, &error)) {
+        g_simple_async_result_take_error (ctx->result, error);
+        sms_store_context_complete_and_free (ctx);
+        return;
+    }
+
+    /* Go on */
     sms_store_next_part (ctx);
 }
 
@@ -296,7 +350,7 @@ send_generic_ready (QmiClientWms *client,
                 &rp_cause,
                 &tp_cause,
                 NULL)) {
-            mm_info ("Couldn't send SMS; RP cause (%u): '%s'; TP cause (%u): '%s'",
+            mm_warn ("Couldn't send SMS; RP cause (%u): '%s'; TP cause (%u): '%s'",
                      rp_cause,
                      qmi_wms_gsm_umts_rp_cause_get_string (rp_cause),
                      tp_cause,
@@ -325,16 +379,29 @@ static void
 sms_send_generic (SmsSendContext *ctx)
 {
     QmiMessageWmsRawSendInput *input;
-    guint8 *pdu;
+    guint8 *pdu = NULL;
     guint pdulen = 0;
     guint msgstart = 0;
     GArray *array;
     GError *error = NULL;
 
     /* Get PDU */
-    pdu = mm_sms_part_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &msgstart, &error);
+    if (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data))
+        pdu = mm_sms_part_3gpp_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &msgstart, &error);
+    else if (MM_SMS_PART_IS_CDMA ((MMSmsPart *)ctx->current->data))
+        pdu = mm_sms_part_cdma_get_submit_pdu ((MMSmsPart *)ctx->current->data, &pdulen, &error);
+
     if (!pdu) {
-        g_simple_async_result_take_error (ctx->result, error);
+        if (error)
+            g_simple_async_result_take_error (ctx->result, error);
+        else
+            g_simple_async_result_set_error (ctx->result,
+                                             MM_CORE_ERROR,
+                                             MM_CORE_ERROR_FAILED,
+                                             "Unknown or unsupported PDU type in SMS part: %s",
+                                             mm_sms_pdu_type_get_string (
+                                                 mm_sms_part_get_pdu_type (
+                                                     (MMSmsPart *)ctx->current->data)));
         sms_send_context_complete_and_free (ctx);
         return;
     }
@@ -346,10 +413,11 @@ sms_send_generic (SmsSendContext *ctx)
     g_free (pdu);
 
     input = qmi_message_wms_raw_send_input_new ();
-
     qmi_message_wms_raw_send_input_set_raw_message_data (
         input,
-        QMI_WMS_MESSAGE_FORMAT_GSM_WCDMA_POINT_TO_POINT,
+        (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data) ?
+         QMI_WMS_MESSAGE_FORMAT_GSM_WCDMA_POINT_TO_POINT :
+         QMI_WMS_MESSAGE_FORMAT_CDMA),
         array,
         NULL);
 
@@ -404,19 +472,39 @@ send_from_storage_ready (QmiClientWms *client,
         } else {
             QmiWmsGsmUmtsRpCause rp_cause;
             QmiWmsGsmUmtsTpCause tp_cause;
+            QmiWmsCdmaCauseCode cdma_cause_code;
+            QmiWmsCdmaErrorClass cdma_error_class;
 
             if (qmi_message_wms_send_from_memory_storage_output_get_gsm_wcdma_cause_info (
                     output,
                     &rp_cause,
                     &tp_cause,
                     NULL)) {
-                mm_info ("Couldn't send SMS; RP cause (%u): '%s'; TP cause (%u): '%s'",
+                mm_warn ("Couldn't send SMS; RP cause (%u): '%s'; TP cause (%u): '%s'",
                          rp_cause,
                          qmi_wms_gsm_umts_rp_cause_get_string (rp_cause),
                          tp_cause,
                          qmi_wms_gsm_umts_tp_cause_get_string (tp_cause));
             }
 
+            if (qmi_message_wms_send_from_memory_storage_output_get_cdma_cause_code (
+                    output,
+                    &cdma_cause_code,
+                    NULL)) {
+                mm_warn ("Couldn't send SMS; cause code (%u): '%s'",
+                         cdma_cause_code,
+                         qmi_wms_cdma_cause_code_get_string (cdma_cause_code));
+            }
+
+            if (qmi_message_wms_send_from_memory_storage_output_get_cdma_error_class (
+                    output,
+                    &cdma_error_class,
+                    NULL)) {
+                mm_warn ("Couldn't send SMS; error class (%u): '%s'",
+                         cdma_error_class,
+                         qmi_wms_cdma_error_class_get_string (cdma_error_class));
+            }
+
             g_prefix_error (&error, "Couldn't write SMS part: ");
             g_simple_async_result_take_error (ctx->result, error);
             sms_send_context_complete_and_free (ctx);
@@ -448,7 +536,9 @@ sms_send_from_storage (SmsSendContext *ctx)
         input,
         mm_sms_storage_to_qmi_storage_type (mm_sms_get_storage (ctx->self)),
         mm_sms_part_get_index ((MMSmsPart *)ctx->current->data),
-        QMI_WMS_MESSAGE_MODE_GSM_WCDMA,
+        (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data) ?
+         QMI_WMS_MESSAGE_MODE_GSM_WCDMA :
+         QMI_WMS_MESSAGE_MODE_CDMA),
         NULL);
 
     qmi_client_wms_send_from_memory_storage (
@@ -485,6 +575,7 @@ sms_send (MMSms *self,
 {
     SmsSendContext *ctx;
     QmiClient *client = NULL;
+    GError *error = NULL;
 
     /* Ensure WMS client */
     if (!ensure_qmi_client (MM_SMS_QMI (self),
@@ -507,7 +598,15 @@ sms_send (MMSms *self,
     /* If the SMS is STORED, try to send from storage */
     ctx->from_storage = (mm_sms_get_storage (self) != MM_SMS_STORAGE_UNKNOWN);
 
-    ctx->current = mm_sms_get_parts (self);;
+    ctx->current = mm_sms_get_parts (self);
+
+    /* Check whether we support the given SMS type */
+    if (!check_sms_type_support (MM_SMS_QMI (self), ctx->modem, (MMSmsPart *)ctx->current->data, &error)) {
+        g_simple_async_result_take_error (ctx->result, error);
+        sms_send_context_complete_and_free (ctx);
+        return;
+    }
+
     sms_send_next_part (ctx);
 }
 
@@ -610,6 +709,12 @@ delete_next_part (SmsDeletePartsContext *ctx)
         input,
         (guint32)mm_sms_part_get_index ((MMSmsPart *)ctx->current->data),
         NULL);
+    qmi_message_wms_delete_input_set_message_mode (
+        input,
+        (MM_SMS_PART_IS_3GPP ((MMSmsPart *)ctx->current->data) ?
+         QMI_WMS_MESSAGE_MODE_GSM_WCDMA:
+         QMI_WMS_MESSAGE_MODE_CDMA),
+        NULL);
     qmi_client_wms_delete (ctx->client,
                            input,
                            5,
