@@ -439,11 +439,7 @@ test_pty_cleanup (gpointer user_data)
     }
 }
 
-#if GLIB_CHECK_VERSION(2,25,12)
 typedef GTestFixtureFunc TCFunc;
-#else
-typedef void (*TCFunc)(void);
-#endif
 
 #define TESTCASE(t, d) g_test_create_case (#t, 0, d, NULL, (TCFunc) t, NULL)
 #define TESTCASE_PTY(t, d) g_test_create_case (#t, sizeof (*d), d, (TCFunc) test_pty_create, (TCFunc) t, (TCFunc) test_pty_cleanup)
@@ -455,7 +451,17 @@ _mm_log (const char *loc,
          const char *fmt,
          ...)
 {
+#if defined ENABLE_TEST_MESSAGE_TRACES
     /* Dummy log function */
+    va_list args;
+    gchar *msg;
+
+    va_start (args, fmt);
+    msg = g_strdup_vprintf (fmt, args);
+    va_end (args);
+    g_print ("%s\n", msg);
+    g_free (msg);
+#endif
 }
 
 int main (int argc, char **argv)
